@@ -1,7 +1,7 @@
 -module(diameter_dict_parser).
 -export([parse/1, parse_and_scan/1, format_error/1]).
 
--file("/build/erlang-nox/src/otp/bootstrap/lib/parsetools/include/yeccpre.hrl", 0).
+-file("/build/erlang-nox-20/src/otp/bootstrap/lib/parsetools/include/yeccpre.hrl", 0).
 %%
 %% %CopyrightBegin%
 %%
